@@ -78,9 +78,7 @@ class Server(object):
         self.environ = {}
         self.handlers = {}
 
-        self._binary_packet = None
-        self._attachment_count = 0
-        self._attachments = []
+        self._binary_packet = []
 
         if not isinstance(logger, bool):
             self.logger = logger
@@ -99,6 +97,8 @@ class Server(object):
         self.manager = client_manager
         self.manager_initialized = False
 
+        self.async_mode = self.eio.async_mode
+
     def on(self, event, handler=None, namespace=None):
         """Register an event handler.
 
@@ -136,6 +136,8 @@ class Server(object):
         client's acknowledgement callback function if it exists. The
         ``'disconnect'`` handler does not take a second argument.
         """
+        if '-' in event:
+            raise ValueError('event names cannot contain hypens')
         namespace = namespace or '/'
 
         def set_handler(handler):
@@ -306,7 +308,31 @@ class Server(object):
         return self.eio.handle_request(environ, start_response)
 
     def start_background_task(self, target, *args, **kwargs):
-        self.eio.start_background_task(target, *args, **kwargs)
+        """Start a background task using the appropriate async model.
+
+        This is a utility function that applications can use to start a
+        background task using the method that is compatible with the
+        selected async mode.
+
+        :param target: the target function to execute.
+        :param args: arguments to pass to the function.
+        :param kwargs: keyword arguments to pass to the function.
+
+        This function returns an object compatible with the `Thread` class in
+        the Python standard library. The `start()` method on this object is
+        already called by this function.
+        """
+        return self.eio.start_background_task(target, *args, **kwargs)
+
+    def sleep(self, seconds=0):
+        """Sleep for the requested amount of time using the appropriate async
+        model.
+
+        This is a utility function that applications can use to put a task to
+        sleep without having to worry about using the correct call for the
+        selected async mode.
+        """
+        return self.eio.sleep(seconds)
 
     def _emit_internal(self, sid, event, data, namespace=None, id=None):
         """Send a message to a client."""
@@ -358,8 +384,8 @@ class Server(object):
             namespace_list = [namespace]
         for n in namespace_list:
             if n != '/' and self.manager.is_connected(sid, n):
-                self.manager.disconnect(sid, n)
                 self._trigger_event('disconnect', n, sid)
+                self.manager.disconnect(sid, n)
         if namespace == '/' and self.manager.is_connected(sid, namespace):
             self._trigger_event('disconnect', '/', sid)
             self.manager.disconnect(sid, '/')
@@ -406,22 +432,14 @@ class Server(object):
 
     def _handle_eio_message(self, sid, data):
         """Dispatch Engine.IO messages."""
-        if self._attachment_count > 0:
-            self._attachments.append(data)
-            self._attachment_count -= 1
-
-            if self._attachment_count == 0:
-                self._binary_packet.reconstruct_binary(self._attachments)
-                if self._binary_packet.packet_type == packet.BINARY_EVENT:
-                    self._handle_event(sid, self._binary_packet.namespace,
-                                       self._binary_packet.id,
-                                       self._binary_packet.data)
+        if len(self._binary_packet):
+            pkt = self._binary_packet[0]
+            if pkt.add_attachment(data):
+                self._binary_packet.pop(0)
+                if pkt.packet_type == packet.BINARY_EVENT:
+                    self._handle_event(sid, pkt.namespace, pkt.id, pkt.data)
                 else:
-                    self._handle_ack(sid, self._binary_packet.namespace,
-                                     self._binary_packet.id,
-                                     self._binary_packet.data)
-                self._binary_packet = None
-                self._attachments = []
+                    self._handle_ack(sid, pkt.namespace, pkt.id, pkt.data)
         else:
             pkt = packet.Packet(encoded_packet=data)
             if pkt.packet_type == packet.CONNECT:
@@ -434,9 +452,7 @@ class Server(object):
                 self._handle_ack(sid, pkt.namespace, pkt.id, pkt.data)
             elif pkt.packet_type == packet.BINARY_EVENT or \
                     pkt.packet_type == packet.BINARY_ACK:
-                self._binary_packet = pkt
-                self._attachments = []
-                self._attachment_count = pkt.attachment_count
+                self._binary_packet.append(pkt)
             elif pkt.packet_type == packet.ERROR:
                 raise ValueError('Unexpected ERROR packet.')
             else:
